package com.inspur.fabric;

import com.inspur.fabric.config.BaasProperties;
import com.inspur.fabric.config.PropertiesConfigService;
import com.inspur.fabric.core.chaincode.DataChaincode;
import com.inspur.fabric.core.factory.DefaultChaincodeFactory;
import com.inspur.fabric.core.factory.DefaultIdentityEntityFactory;
import lombok.extern.slf4j.Slf4j;

/**
 * @author zhanglan
 * @description
 * @date 2021/8/13
 */
@Slf4j
public class Main {

    // 通道名称
    private static final String CHANNEL_NAME = "test-wn001";
    // 链码名称
    private static final String CC_NAME = "lianma-wn001";
    // 链码版本
    private static final String CC_VER = "1.1";

    // 成员名称
    private static final String ORG_NAME = "org-70364us3d3";
    // 成员的mspid
    private static final String MSP_ID = "org-70364us3d3MSP";

    // 成员详情中注册的ca用户的用户名
    private static final String IDENTITY_ID = "fogray";
    // 成员详情中注册的ca用户的证书
    private static final String CERT = "-----BEGIN CERTIFICATE-----\r\n" + 
    		"MIIC7TCCApOgAwIBAgIUWoTvWq7lWFaIvbV2M+wfkIlpBCgwCgYIKoZIzj0EAwIw\r\n" + 
    		"ga8xCzAJBgNVBAYTAkNOMREwDwYDVQQIEwhTaGFuRG9uZzEOMAwGA1UEBxMFSmlO\r\n" + 
    		"YW4xPDA6BgNVBAoTM29yZy03MDM2NHVzM2QzLjAwMzRlYzQ0LTJhNDAtNDk1Ny1i\r\n" + 
    		"NjIwLTNkZDYwYmNiNjNhMzE/MD0GA1UEAxM2Y2Eub3JnLTcwMzY0dXMzZDMuMDAz\r\n" + 
    		"NGVjNDQtMmE0MC00OTU3LWI2MjAtM2RkNjBiY2I2M2EzMB4XDTIxMDgxOTAyMTEw\r\n" + 
    		"MFoXDTIyMDgxOTAyMTYwMFowLTEaMAsGA1UECxMEdXNlcjALBgNVBAsTBG9yZzEx\r\n" + 
    		"DzANBgNVBAMTBmZvZ3JheTBZMBMGByqGSM49AgEGCCqGSM49AwEHA0IABPZSDAlO\r\n" + 
    		"i8syc8WnLhvGckqILbPPHowii3G3c8YHRrIKvL/IWQ8WFxsAyNfpza71SdWQ85GN\r\n" + 
    		"uZ7etkQ58U7UC6KjggEMMIIBCDAOBgNVHQ8BAf8EBAMCB4AwDAYDVR0TAQH/BAIw\r\n" + 
    		"ADAdBgNVHQ4EFgQUAiPkdqMXn/neU8wNC4lbG952XaowKwYDVR0jBCQwIoAgy8py\r\n" + 
    		"2fNlOkKxXzFVf4E8kZeF7DSuL/rv2xWS1fArXCYwPwYDVR0RBDgwNoI0YWNzLTk3\r\n" + 
    		"MjJhMWIxLTQ4YTAtNDQ1NS05NmM4LWY4ZjMwYmMyZWFjYy0zLm5vdmFsb2NhbDBb\r\n" + 
    		"BggqAwQFBgcIAQRPeyJhdHRycyI6eyJoZi5BZmZpbGlhdGlvbiI6Im9yZzEiLCJo\r\n" + 
    		"Zi5FbnJvbGxtZW50SUQiOiJmb2dyYXkiLCJoZi5UeXBlIjoidXNlciJ9fTAKBggq\r\n" + 
    		"hkjOPQQDAgNIADBFAiEAtlF+7y+fF+Yl/+hngcMC59cPRK8o6BCv7C4D7YaCjwsC\r\n" + 
    		"IDajOXX86NeKJ5DW3LPVig8YBXq3L93HWNxDTeOmhTDT\r\n" + 
    		"-----END CERTIFICATE-----";
    // 成员详情中注册的ca用户的私钥
    private static final String PRV_KEY = "-----BEGIN PRIVATE KEY-----\r\n" + 
    		"MIGHAgEAMBMGByqGSM49AgEGCCqGSM49AwEHBG0wawIBAQQg9BNiQD1FY7TraZx5\r\n" + 
    		"cT8wShIYWT27/0HAODl0GoUX2rOhRANCAAT2UgwJTovLMnPFpy4bxnJKiC2zzx6M\r\n" + 
    		"Iotxt3PGB0ayCry/yFkPFhcbAMjX6c2u9UnVkPORjbme3rZEOfFO1Aui\r\n" + 
    		"-----END PRIVATE KEY-----\r\n" + 
    		"";

    /**
     * 在acs中成功完成创建联盟、成员、通道及安装部署链码等操作后，进行如下操作以初始化sdk
     * 1、下载联盟证书和成员证书，将下载好的联盟证书与成员证书解压到channel/crypto-config文件夹下，目录结构参考示例channel文件夹
     * 2、根据联盟和成员相关信息，修改fabric.properties文件，内容参考示例文件
     * 3、将channel文件夹路径和fabric.properties配置文件路径作为参数传入传入sdk中
     * 4、根据具体的链码实例化链码实例
     * @param args
     */
    public static void main(String[] args) {
        // 将channel文件夹路径和fabric.properties配置文件路径配置到BaasProperties中
        BaasProperties baasProperties = new BaasProperties();
        baasProperties.setChannelPath("D:\\gitlab_inspur\\acs-deploy\\fabric-sdk-demo\\src\\main\\resources\\channel");
        baasProperties.setConfigFilePath("D:\\gitlab_inspur\\acs-deploy\\fabric-sdk-demo\\src\\main\\resources\\fabric.properties");

        // 传入BaasProperties创建sdk的配置服务PropertiesConfigService
        PropertiesConfigService configService = new PropertiesConfigService(baasProperties);
        //根据PropertiesConfigService创建默认的链码工厂DefaultChaincodeFactory
        DefaultChaincodeFactory instance = DefaultChaincodeFactory.getInstance(configService);

        // 根据ca用户的名称、私钥、证书等创建用户身份实体
        DefaultIdentityEntityFactory.getInstance(configService).createIdentityEntity(ORG_NAME, IDENTITY_ID, MSP_ID, PRV_KEY, CERT);

        // 根据通道名称、ca用户名、和具体的链码调用实体类创建链码调用实体类实例
        DataChaincode chaincode = instance.getChaincode(CHANNEL_NAME, CC_NAME, CC_VER, IDENTITY_ID, DataChaincode.class);

        // 调用链码
        String result = chaincode.invokeCc("invoke", new String[] {"a", "b", "2"});
        log.info("result1: {}", result);

        // 调用链码
        result = chaincode.queryCc("query", new String[] {"a"});
        log.info("result2: {}", result);
    }

}
