package com.inspur.fabric.core.chaincode;

import static org.apache.commons.codec.CharEncoding.UTF_8;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.CompletableFuture;

import org.hyperledger.fabric.sdk.ChaincodeID;
import org.hyperledger.fabric.sdk.Peer;
import org.hyperledger.fabric.sdk.ProposalResponse;
import org.hyperledger.fabric.sdk.QueryByChaincodeRequest;
import org.hyperledger.fabric.sdk.SDKUtils;
import org.hyperledger.fabric.sdk.TransactionProposalRequest;

import com.inspur.fabric.core.entity.ChannelEntity;

import lombok.extern.slf4j.Slf4j;

/**
 * @author zhanglan
 * @description
 * @date 2021/8/13
 */
@Slf4j
public abstract class AbstractChaincode implements IChaincode {
    protected ChannelEntity channel;

    protected ChaincodeID chaincodeID;

    protected abstract void init(ChannelEntity channel);

    @Override
    public Map<String, Object> invoke(String funcName, String[] args) {
        Map<String, Object> result = new HashMap<>();
        String resultAsString = null;
        try {
            Collection<ProposalResponse> successful = new LinkedList<>();
            Collection<ProposalResponse> failed = new LinkedList<>();
            TransactionProposalRequest transactionProposalRequest = channel.getClient().newTransactionProposalRequest();
            transactionProposalRequest.setChaincodeID(chaincodeID);
            transactionProposalRequest.setFcn(funcName);
            transactionProposalRequest.setArgs(args);

            Map<String, byte[]> tm = new HashMap<>();
            tm.put("HyperLedgerFabric", "TransactionProposalRequest:JavaSDK".getBytes(UTF_8));
            tm.put("method", "TransactionProposalRequest".getBytes(UTF_8));
            tm.put("result", "success".getBytes(UTF_8)); /// This should be returned see
            transactionProposalRequest.setTransientMap(tm);

            Collection<Peer> endorsers = channel.getChannel().getPeers();
            Collection<ProposalResponse> transactionPropResp = channel.getChannel().sendTransactionProposal(transactionProposalRequest,
                    endorsers);
            for (ProposalResponse response : transactionPropResp) {
                if (response.getStatus() == ProposalResponse.Status.SUCCESS) {
                    successful.add(response);
                    resultAsString = new String(response.getChaincodeActionResponsePayload(), "UTF-8");
                    result.put("result", resultAsString);
                    result.put("tx_id", response.getTransactionID());
                } else {
                    failed.add(response);
                }
            }

            Collection<Set<ProposalResponse>> proposalConsistencySets = SDKUtils.getProposalConsistencySets(transactionPropResp);
            if (proposalConsistencySets.size() != 1) {
                throw new RuntimeException("背书提案响应一致性校验失败");
            }
            CompletableFuture<String> cf = channel.getChannel().sendTransaction(successful).thenApply(event -> {
                return event.getTransactionID();
            }).exceptionally(e -> {
                throw new RuntimeException("invoke error:", e);
            });
        } catch (Exception e) {
            log.info("AbstractChaincode invoke error: ", e);
        }
        return result;
    }

    @Override
    public String query(String funcName, String[] args) {
        String resultAsString = "";
        try {
            QueryByChaincodeRequest request = channel.getClient().newQueryProposalRequest();
            request.setChaincodeID(chaincodeID);
            request.setFcn(funcName);
            request.setArgs(args);

            Map<String, byte[]> tm = new HashMap<>();
            tm.put("HyperLedgerFabric", "TransactionProposalRequest:JavaSDK".getBytes(UTF_8));
            tm.put("method", "TransactionProposalRequest".getBytes(UTF_8));
            tm.put("result", "success".getBytes(UTF_8));
            request.setTransientMap(tm);

            Collection<ProposalResponse> queryProposals = channel.getChannel().queryByChaincode(request, channel.getChannel().getPeers());

            for (ProposalResponse response : queryProposals) {
                if (response.getStatus() == ProposalResponse.Status.SUCCESS) {
                    resultAsString = new String(response.getChaincodeActionResponsePayload(), "UTF-8");
                }
            }
            if (resultAsString == null) {
                resultAsString = "";
            }
        } catch (Exception e) {
            throw new RuntimeException("读链错误", e);
        }

        return resultAsString;
    }

}
