package com.inspur.fabric.core.entity;

import lombok.Data;
import org.hyperledger.fabric.sdk.Channel;
import org.hyperledger.fabric.sdk.HFClient;
import org.hyperledger.fabric.sdk.Peer;
import org.hyperledger.fabric.sdk.User;
import org.hyperledger.fabric.sdk.security.CryptoSuite;

import java.util.Collection;
import java.util.EnumSet;

/**
 * @author zhanglan
 * @description
 * @date 2021/8/13
 */
@Data
public class ChannelEntity {

    private final String channelName;

    private final User userContext;

    private final HFClient client;

    private final Collection<PeerEntity> peers;

    private final Collection<OrdererEntity> orderers;

    private final Channel channel;

    private ChannelEntity(String channelName, User userContext, Collection<PeerEntity> peers, Collection<OrdererEntity> orderers) {
        this.channelName = channelName;
        this.userContext = userContext;
        this.peers = peers;
        this.orderers = orderers;
        client = HFClient.createNewInstance();
        try {
            client.setCryptoSuite(CryptoSuite.Factory.getCryptoSuite());
            client.setUserContext(userContext);
            channel = client.newChannel(channelName);
//			channel = new ChannelImp(channelName, client);
        } catch (Exception e) {
            throw new RuntimeException("construct ChannelEntity wrong:", e);
        }
    }

    private void initialize() {
        try {
            for (OrdererEntity entity : orderers) {
                channel.addOrderer(client.newOrderer(entity.getOrdererName(), entity.getOrdererLocation(), entity.getOrdererProperties()));
            }
            for (PeerEntity entity : peers) {
                Peer peer = client.newPeer(entity.getPeerName(), entity.getPeerLocation(), entity.getPeerProperties());
                channel.addPeer(peer, Channel.PeerOptions.createPeerOptions().setPeerRoles(EnumSet.of(Peer.PeerRole.ENDORSING_PEER, Peer.PeerRole.LEDGER_QUERY, Peer.PeerRole.CHAINCODE_QUERY, Peer.PeerRole.EVENT_SOURCE)));
            }
            if (!channel.isInitialized()) {
                channel.initialize();
            }
        } catch (Exception e) {
            throw new RuntimeException("initialize ChannelEntity wrong:", e);
        }
    }

    public static Builder newBuilder() {
        return new Builder();
    }

    public static class Builder {
        private String channelName;

        private User userContext;

        private Collection<PeerEntity> peers;

        private Collection<OrdererEntity> orderers;

        private Builder() {
        }

        public Builder setChannelName(String channelName) {
            this.channelName = channelName;
            return this;
        }

        public Builder setUserContext(User userContext) {
            this.userContext = userContext;
            return this;
        }

        public Builder setPeers(Collection<PeerEntity> peers) {
            this.peers = peers;
            return this;
        }

        public Builder setOrderers(Collection<OrdererEntity> orderers) {
            this.orderers = orderers;
            return this;
        }

        public ChannelEntity build() {
            ChannelEntity entity = new ChannelEntity(channelName, userContext, peers, orderers);
            entity.initialize();
            return entity;
        }

    }

}
