package com.inspur.fabric.core.entity;

import lombok.Data;
import org.bouncycastle.util.encoders.Hex;
import org.hyperledger.fabric.sdk.Enrollment;

import javax.security.cert.X509Certificate;
import java.security.KeyFactory;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.spec.PKCS8EncodedKeySpec;

/**
 * @author zhanglan
 * @description
 * @date 2021/8/13
 */
@Data
public class EnrollmentEntity implements Enrollment {
    private String prvKeyHex;

    private String pubKeyHex;

    private PrivateKey key;

    private String cert;

    public EnrollmentEntity(Enrollment enrollment) {
        setKey(enrollment.getKey());
        setCert(enrollment.getCert());
        setPrvKeyHex(String.valueOf(Hex.encode(enrollment.getKey().getEncoded())));
        setPubKeyHex(getPubKeyFromCert(enrollment.getCert()));
    }

    public EnrollmentEntity(String prvKeyHex, String cert) {
        setPrvKeyHex(prvKeyHex);
        setPubKeyHex(getPubKeyFromCert(cert));
        setKey(getPrivateKey(prvKeyHex));
        setCert(cert);
    }

    public EnrollmentEntity(PrivateKey prvKey, String cert) {
        setKey(prvKey);
        setCert(cert);
        setPrvKeyHex(String.valueOf(Hex.encode(prvKey.getEncoded())));
        setPubKeyHex(getPubKeyFromCert(cert));
    }

    private PrivateKey getPrivateKey(String key) {
        PrivateKey pk = null;
        try {
            byte[] keyBytes = Hex.decode(key);
            PKCS8EncodedKeySpec keySpec = new PKCS8EncodedKeySpec(keyBytes);
            KeyFactory keyFactory = KeyFactory.getInstance("EC");
            pk = keyFactory.generatePrivate(keySpec);
        } catch (Exception e) {
            throw new RuntimeException("BaasEnrollment getPubKeyFromCert error key:" + key, e);
        }
        return pk;
    }

    private String getPubKeyFromCert(String certStr) {
        String publicKeyStr = "";
        try {
            X509Certificate cert = X509Certificate.getInstance(certStr.getBytes());
//			X509Certificate cert = SM2CertUtil.getX509Certificate(certStr.getBytes());
            PublicKey publicKey = cert.getPublicKey();
            publicKeyStr = String.valueOf(Hex.encode(publicKey.getEncoded()));
        } catch (Exception e) {
            throw new RuntimeException("BaasEnrollment getPubKeyFromCert error cert:" + certStr, e);
        }
        return publicKeyStr;
    }
}
