package com.inspur.fabric.core.factory;


import com.inspur.fabric.config.IConfigService;
import com.inspur.fabric.core.chaincode.IChaincode;
import com.inspur.fabric.core.entity.ChannelEntity;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author zhanglan
 * @description
 * @date 2021/8/13
 */
public class DefaultChaincodeFactory implements IChaincodeFactory {
    private static volatile DefaultChaincodeFactory instance;

    private DefaultFabricEntityFactory factory;

    public static DefaultChaincodeFactory getInstance(IConfigService config) {
        if (instance == null) {
            synchronized (DefaultFabricEntityFactory.class) {
                if (instance == null) {
                    instance = new DefaultChaincodeFactory(config);
                }
            }
        }
        return instance;
    }

    private DefaultChaincodeFactory(IConfigService config) {
        factory = DefaultFabricEntityFactory.getInstance(config);
    }

    private ConcurrentHashMap<Class, ConcurrentHashMap<String, ConcurrentHashMap<String, IChaincode>>> chaincodeCache = new ConcurrentHashMap<>();

    @Override
    public <T extends IChaincode> T getChaincode(String channelName, String ccName, String ccVer, String identityId, Class<T> clazz) {
        if (!chaincodeCache.containsKey(clazz)) {
            chaincodeCache.put(clazz, new ConcurrentHashMap<String, ConcurrentHashMap<String, IChaincode>>());
        }
        if (!chaincodeCache.get(clazz).containsKey(channelName)) {
            chaincodeCache.get(clazz).put(channelName, new ConcurrentHashMap<String, IChaincode>());
        }

        if (!chaincodeCache.get(clazz).get(channelName).containsKey(identityId)) {
            T chaincode = createChaincode(channelName, ccName, ccVer, identityId, clazz);
            chaincodeCache.get(clazz).get(channelName).put(identityId, chaincode);
        }

        return (T) chaincodeCache.get(clazz).get(channelName).get(identityId);
    }

    public <T extends IChaincode> T createChaincode(String channelName, String ccName, String ccVer, String identityId, Class<T> clazz) {
        try {
            ChannelEntity channel = null;
            if (identityId.startsWith("peerOrg")) {
                channel = factory.getPeerAdminChannelFromCacheOrPut(channelName, identityId);
            } else {
                channel = factory.getCommonChannelFromCacheOrPut(channelName, identityId);
            }
            Constructor<T> constructor = clazz.getDeclaredConstructor(ChannelEntity.class, String.class, String.class);
            constructor.setAccessible(true);
            T chaincode = constructor.newInstance(channel, ccName, ccVer);
            return chaincode;
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        }
        return null;
    }

}
