package com.inspur.fabric.core.factory;

import com.inspur.fabric.config.IConfigService;
import com.inspur.fabric.core.entity.*;
import org.hyperledger.fabric.sdk.User;

import java.util.ArrayList;
import java.util.Collection;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author zhanglan
 * @description
 * @date 2021/8/13
 */
public class DefaultFabricEntityFactory {
    private static volatile DefaultFabricEntityFactory instance;

    private IIdentityEntityFactory identityEntityFactory;

    public static DefaultFabricEntityFactory getInstance(IConfigService config) {
        if (instance == null) {
            synchronized (DefaultFabricEntityFactory.class) {
                if (instance == null) {
                    instance = new DefaultFabricEntityFactory(config);
                }
            }
        }
        return instance;
    }

    private IConfigService config;

    private DefaultFabricEntityFactory(IConfigService config) {
        this.config = config;
        this.identityEntityFactory = DefaultIdentityEntityFactory.getInstance(config);
        initializeOrgEntitys();
//        initializeBaseChannelEntitys();
    }

    private ConcurrentHashMap<String, OrgEntity> orgEntityMap = new ConcurrentHashMap<String, OrgEntity>();

    private ConcurrentHashMap<String, PeerEntity> peerEntityMap = new ConcurrentHashMap<String, PeerEntity>();

    private ConcurrentHashMap<String, OrdererEntity> ordererEntityMap = new ConcurrentHashMap<String, OrdererEntity>();

    private ConcurrentHashMap<String, User> adminMap = new ConcurrentHashMap<String, User>();

    private ConcurrentHashMap<String, User> peerAdminMap = new ConcurrentHashMap<String, User>();

    private ConcurrentHashMap<String, ConcurrentHashMap<String, ChannelEntity>> peerAdminChannelCache = new ConcurrentHashMap<String, ConcurrentHashMap<String, ChannelEntity>>();

    private ConcurrentHashMap<String, ConcurrentHashMap<String, ChannelEntity>> commonChannelCache = new ConcurrentHashMap<String, ConcurrentHashMap<String, ChannelEntity>>();

    private void initializeOrgEntitys() {
        Collection<OrgEntity> orgs = config.listBaseOrgInfo();
        for (OrgEntity org : orgs) {
            orgEntityMap.put(org.getOrgName(), org);
            for (PeerEntity peer : org.getPeers()) {
                peerEntityMap.put(peer.getPeerName(), peer);
            }
            adminMap.put(org.getOrgName(), org.getAdmin());
            peerAdminMap.put(org.getOrgName(), org.getPeerAdmin());
        }

        Collection<OrdererEntity> orderers = config.listBaseOrdererInfo();
        for (OrdererEntity orderer : orderers) {
            ordererEntityMap.put(orderer.getOrdererName(), orderer);
        }

    }

    private void initializeBaseChannelEntitys() {
        String orgName = "peerOrg1";
        String channelName = "trace1";
        getPeerAdminChannelFromCacheOrPut(channelName, orgName);

    }

    private Collection<PeerEntity> listPeerOfChannel(String channelName) {
        return config.listPeerOfChannel(channelName);
    }

    public ChannelEntity getPeerAdminChannelFromCacheOrPut(String channelName, String orgName) {
        if (!peerAdminChannelCache.containsKey(channelName)) {
            peerAdminChannelCache.put(channelName, new ConcurrentHashMap<String, ChannelEntity>());
        }
        if (!peerAdminChannelCache.get(channelName).containsKey(orgName)) {
            OrgEntity org = orgEntityMap.get(orgName);
            User peerAdmin = org.getPeerAdmin();
            Collection<PeerEntity> peers = listPeerOfChannel(channelName);
            Collection<OrdererEntity> orderers = new ArrayList<OrdererEntity>(ordererEntityMap.values());
            ChannelEntity channel = ChannelEntity.newBuilder().setChannelName(channelName)
                    .setUserContext(peerAdmin).setPeers(peers).setOrderers(orderers).build();
            peerAdminChannelCache.get(channelName).put(orgName, channel);
        }
        return peerAdminChannelCache.get(channelName).get(orgName);
    }

    public ChannelEntity getCommonChannelFromCacheOrPut(String channelName, String identityId) {
        if (!commonChannelCache.containsKey(channelName)) {
            commonChannelCache.put(channelName, new ConcurrentHashMap<String, ChannelEntity>());
        }
        if (!commonChannelCache.get(channelName).containsKey(identityId)) {
            Collection<OrdererEntity> orderers = new ArrayList<OrdererEntity>(ordererEntityMap.values());
            ChannelEntity channel = ChannelEntity.newBuilder().setChannelName(channelName)
                    .setUserContext(identityEntityFactory.getIdentityEntity(identityId))
                    .setPeers(listPeerOfChannel(channelName))
                    .setOrderers(orderers)
                    .build();
            commonChannelCache.get(channelName).put(identityId, channel);
        }
        return commonChannelCache.get(channelName).get(identityId);
    }


}
