package com.inspur.fabric.core.factory;

import com.inspur.fabric.config.IConfigService;
import com.inspur.fabric.core.entity.AbstractIdentityEntity;
import com.inspur.fabric.core.entity.CommonIdentityEntity;
import com.inspur.fabric.core.entity.EnrollmentEntity;
import com.inspur.fabric.core.util.CertUtil;

import java.util.concurrent.ConcurrentHashMap;

/**
 * @author zhanglan
 * @description
 * @date 2021/8/13
 */
public class DefaultIdentityEntityFactory implements IIdentityEntityFactory {
    private static volatile DefaultIdentityEntityFactory instance;

    private DefaultIdentityEntityFactory(IConfigService config) {
        this.config = config;
    }

    public static DefaultIdentityEntityFactory getInstance(IConfigService config) {
        if (instance == null) {
            synchronized (DefaultIdentityEntityFactory.class) {
                if (instance == null) {
                    instance = new DefaultIdentityEntityFactory(config);
                }
            }
        }
        return instance;
    }

    private IConfigService config;


    private ConcurrentHashMap<String, AbstractIdentityEntity> identityEntityMap = new ConcurrentHashMap<String, AbstractIdentityEntity>();

    @Override
    public AbstractIdentityEntity getIdentityEntity(String identityId) {
        return identityEntityMap.get(identityId);
    }

    @Override
    public AbstractIdentityEntity createIdentityEntity(String orgName, String identityId, String mspId, String prvKey, String cert) {
        String prvKeyHex = CertUtil.getPrivateKeyFromCert(prvKey);
        EnrollmentEntity enrollmentEntity = new EnrollmentEntity(prvKeyHex, cert);
        CommonIdentityEntity entity = CommonIdentityEntity.newBuilder().setIdentityId(identityId).setOrgName(orgName).setMspId(mspId).setEnrollment(enrollmentEntity).build();
        identityEntityMap.put(identityId, entity);
        return entity;
    }

    public void putIdentityEntity(AbstractIdentityEntity entity) {
        identityEntityMap.put(entity.getName(), entity);
    }


}
