package com.inspur.fabric.core.util;

import com.google.protobuf.InvalidProtocolBufferException;
import com.inspur.fabric.core.entity.ChannelEntity;
import lombok.extern.slf4j.Slf4j;
import org.bouncycastle.util.encoders.Hex;
import org.hyperledger.fabric.sdk.BlockInfo;
import org.hyperledger.fabric.sdk.TransactionInfo;
import org.hyperledger.fabric.sdk.exception.InvalidArgumentException;
import org.hyperledger.fabric.sdk.exception.ProposalException;

import java.util.HashMap;
import java.util.Map;

/**
 * @author zhanglan
 * @description
 * @date 2021/8/13
 */
@Slf4j
public class BlockUtil {

    private BlockUtil() {
        if (BlockUtilHolder.instance != null) {
            throw new RuntimeException("只允许单例");
        }
    }

    public static BlockUtil getInstance() {
        return BlockUtilHolder.instance;
    }

    public Map<String, Object> getBlockInfoByTxId(ChannelEntity channelEntity, String txId) {
        try {
//            TransactionInfo transactionInfo = channelEntity.getChannel().queryTransactionByID(txId);
            BlockInfo blockInfo = channelEntity.getChannel().queryBlockByTransactionID(txId);
            long blockNumber = blockInfo.getBlockNumber();
            String blockHash = Hex.toHexString(blockInfo.getDataHash());
            Map<String, Object> map = new HashMap<>();
            map.put("block_num", blockNumber);
            map.put("block_hash", blockHash);
            return map;
        } catch (Exception e) {
            log.error("BlockUtil getTxInfoById error: ", e);
        }
        return null;
    }


    private static class BlockUtilHolder {
        private static BlockUtil instance = new BlockUtil();
    }
}
