package com.inspur.fabric.core.util;

import com.sun.org.apache.xerces.internal.impl.dv.util.HexBin;
import lombok.extern.slf4j.Slf4j;
import org.bouncycastle.asn1.pkcs.PrivateKeyInfo;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.openssl.PEMParser;
import org.bouncycastle.openssl.jcajce.JcaPEMKeyConverter;

import javax.security.cert.X509Certificate;
import java.io.Reader;
import java.io.StringReader;
import java.security.PrivateKey;
import java.security.PublicKey;

/**
 * @author zhanglan
 * @description
 * @date 2021/8/16
 */
@Slf4j
public class CertUtil {
    /**
     * 从证书中获取公钥
     * @param certStr
     * @return
     */
    public static String getPublicKeyFromCert(String certStr) {
        String publicKeyStr = "";
        try {
            X509Certificate cert = X509Certificate.getInstance(certStr.getBytes());
            PublicKey publicKey = cert.getPublicKey();
            publicKeyStr = HexBin.encode(publicKey.getEncoded());
        } catch (Exception e) {
            log.error("getPublicKeyFromCert error:", e);
        }
        return publicKeyStr;
    }

    /**
     * 从pem证书中获取私钥
     * @param certStr
     * @return
     */
    public static String getPrivateKeyFromCert(String certStr) {
        try {
            final Reader pemReader = new StringReader(certStr);
            final PrivateKeyInfo pemPair;
            try (PEMParser pemParser = new PEMParser(pemReader)) {
                pemPair = (PrivateKeyInfo) pemParser.readObject();
            }
            PrivateKey privateKey = new JcaPEMKeyConverter().setProvider(new BouncyCastleProvider()).getPrivateKey(pemPair);
            return HexBin.encode(privateKey.getEncoded());
        } catch (Exception e) {
            log.error("getPrivateKeyFromCert error:", e);
        }
        return null;
    }

}
